USE [igops]
GO

DROP VIEW IF EXISTS user_perm_delta_v
DROP VIEW IF EXISTS suser_v
DROP VIEW IF EXISTS suser_brole_v
DROP VIEW IF EXISTS user_perm_snapshot_v
DROP VIEW IF EXISTS spermission_v
DROP VIEW IF EXISTS spermission_all_v
GO

CREATE VIEW suser_v AS
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk
  FROM suser
    LEFT JOIN (SELECT DISTINCT 
        suser.id suser,
        suser.display_name,
        STUFF((SELECT DISTINCT ' | ' + suser_email.email
          FROM suser suser1
            JOIN suser_email ON suser.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
          WHERE suser1.id = suser.id
          FOR XML PATH('')),1,3,'') email
      FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = 1 AND suser.deleted = 0
      GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
  WHERE suser.deleted = 0 AND suser.effective = 1

GO

CREATE VIEW user_perm_snapshot_v AS
	SELECT DISTINCT
		appl.qid AS appl_qid,
		ds.name AS appl_name,
		perm.unique_permission_id AS perm_qid,
		perm.name AS perm_name,
		perm.type,
		usr.unique_user_id AS user_qid,
		usr.dn AS user_dn,
		usr.last_name AS user_last_name,
		usr.first_name AS user_first_name,
		usr.idm_tree_name AS user_idm_tree_name,
		snap.id AS snap_id,
		snap.[timestamp] AS snap_timestamp
	FROM
		spermission perm
		LEFT JOIN application appl ON perm.application = appl.id
		LEFT JOIN spermission pcoll ON perm.collected_entity = pcoll.id
		LEFT JOIN saccount_permission ap ON	pcoll.id = ap.spermission
		LEFT JOIN saccount acoll ON	ap.saccount = acoll.id
		LEFT JOIN saccount_user au ON acoll.id = au.saccount
		LEFT JOIN saccount acct ON acoll.id = acct.collected_entity
		LEFT JOIN suser ucoll ON au.suser = ucoll.id
		LEFT JOIN suser usr ON ucoll.id = usr.collected_entity
		LEFT JOIN data_source ds ON appl.id = ds.id
		LEFT JOIN snapshot_sversions ap_snap_sv ON ap.sversion = ap_snap_sv.sversions
		LEFT JOIN snapshot_sversions au_snap_sv ON au.sversion = au_snap_sv.sversions
		INNER JOIN snapshot snap ON ap_snap_sv.snapshot = snap.id AND au_snap_sv.snapshot = snap.id

GO

CREATE VIEW spermission_v AS
SELECT
  spermission.id,
  spermission.account_grant,
  spermission.assignable,
  spermission.authoritative,
  spermission.bound_by_id,
  spermission.boundable,
  spermission.cost,
  spermission.deleted,
  spermission.description,
  spermission.display_name,
  spermission.driver_id,
  spermission.effective,
  spermission.exclude_from_catalog,
  spermission.is_curated,
  spermission.logical_id,
  spermission.merged,
  spermission.name,
  spermission.not_reviewable,
  spermission.permission_id,
  spermission.prov_attribute,
  spermission.prov_by_unique_permission_id,
  spermission.prov_native_value,
  spermission.prov_unique_driver_id,
  spermission.provisionable,
  spermission.risk,
  spermission.type,
  spermission.unique_bound_by_id,
  spermission.unique_permission_id,
  spermission.value,
  spermission.value_type,
  spermission.application,
  spermission.base_application,
  spermission.collected_entity,
  spermission.data_collection,
  spermission.effective_entity,
  spermission.sversion,
  spermission.auto_generated,
  spermission.prov_unique_permission_id,
  perm_owner.perm_owner_name
FROM 
  spermission
  LEFT JOIN (SELECT 
        rso.permission,
        STUFF((SELECT DISTINCT ' | ' + suser.display_name FROM resolved_spermission_owner rso1
        JOIN (SELECT unique_user_id, display_name FROM suser WHERE suser.deleted = '0' AND suser.effective = '1') suser ON suser.unique_user_id = rso1.owner_unique_id
        WHERE rso1.permission = rso.permission
        FOR XML PATH('')),1,3,'') perm_owner_name
        FROM (SELECT DISTINCT owner_unique_id, permission
                  FROM resolved_spermission_owner) rso
                  JOIN suser ON suser.unique_user_id = rso.owner_unique_id AND suser.deleted = '0' AND suser.effective = '1'
          GROUP BY rso.permission) perm_owner ON perm_owner.permission = spermission.id
WHERE 
  (exclude_from_catalog = 0 OR exclude_from_catalog IS NULL) AND effective = 1
GO


CREATE VIEW user_perm_delta_v AS
  SELECT DISTINCT pchange.snapshot,
    pchange.unique_permission_id AS perm_qid,
    perm.name AS perm_name,
    old_perm.name AS old_perm_name,
    perm.type,
    old_perm.type AS old_perm_type,
    perm.description,
    old_perm.description AS old_perm_desc,
    perm.risk,
    old_perm.risk AS old_perm_risk,
    perm.value,
    old_perm.value AS old_perm_value,
    perm.id AS permission_id,
    perm.collected_entity,
    pchange.new_permission,
    pchange.old_permission,
    pchange.bound_by_id_changed,
    pchange.change_type,
    pchange.name_changed,
    pchange.description_changed,
    pchange.risk_level_changed,
    pchange.type_changed,
    pchange.value_changed,
    pchange.application,
    ds.unique_id AS appl_qid,
    ds.name AS appl_name,
    usr.unique_user_id user_qid,
    usr.last_name AS user_last_name,
    usr.first_name AS user_first_name,
    usr.idm_dn AS user_dn,
    usr.idm_tree_name AS user_idm_tree_name,
    snap.id AS snap_id,
    snap.[timestamp] AS snap_timestamp
   FROM permission_change AS pchange
     LEFT JOIN spermission AS perm ON pchange.unique_permission_id = perm.unique_permission_id AND perm.collected_entity = pchange.new_permission AND perm.id = perm.collected_entity
     LEFT JOIN spermission AS old_perm ON perm.unique_permission_id = old_perm.unique_permission_id AND old_perm.collected_entity = pchange.old_permission AND old_perm.id = old_perm.collected_entity
     LEFT JOIN data_source AS ds ON perm.application = ds.id
     LEFT JOIN saccount_permission AS ap ON ap.spermission = perm.id
     LEFT JOIN saccount AS acoll ON ap.saccount = acoll.id
     LEFT JOIN saccount_user AS au ON au.saccount = acoll.id
     LEFT JOIN suser AS usr ON usr.collected_entity = au.suser
     LEFT JOIN snapshot AS snap ON pchange.snapshot = snap.id OR pchange.old_snapshot = snap.id OR pchange.change_type = 'REMOVED'
  WHERE snap.id = pchange.snapshot AND (perm.type != 'BoundEntitlement' OR pchange.change_type = 'REMOVED') AND (pchange.change_type = 'ADDED' OR pchange.change_type = 'REMOVED' OR pchange.name_changed = 1 OR pchange.description_changed = 1 OR pchange.risk_level_changed = 1 OR pchange.type_changed = 1 OR pchange.value_changed = 1);
GO

CREATE VIEW spermission_all_v AS
  SELECT
    spermission.id,
    spermission.account_grant,
    spermission.assignable,
    spermission.authoritative,
    spermission.bound_by_id,
    spermission.boundable,
    spermission.cost,
    spermission.deleted,
    spermission.description,
    spermission.display_name,
    spermission.driver_id,
    spermission.effective,
    spermission.exclude_from_catalog,
    spermission.is_curated,
    spermission.logical_id,
    spermission.merged,
    spermission.name,
    spermission.not_reviewable,
    spermission.permission_id,
    spermission.prov_attribute,
    spermission.prov_by_unique_permission_id,
    spermission.prov_native_value,
    spermission.prov_unique_driver_id,
    spermission.provisionable,
    spermission.risk,
    spermission.type,
    spermission.unique_bound_by_id,
    spermission.unique_permission_id,
    spermission.value,
    spermission.value_type,
    spermission.application,
    spermission.base_application,
    spermission.collected_entity,
    spermission.data_collection,
    spermission.effective_entity,
    spermission.sversion,
    spermission.auto_generated,
    spermission.prov_unique_permission_id
  FROM spermission

GO

GRANT SELECT ON user_perm_delta_v TO ig_report_role
GRANT SELECT ON suser_v TO ig_report_role
GRANT SELECT ON user_perm_snapshot_v TO ig_report_role
GRANT SELECT ON spermission_v TO ig_report_role
GRANT SELECT ON spermission_all_v TO ig_report_role
GO