  BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'USER_PERM_DELTA_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM USER_PERM_DELTA_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'USER_PERM_DELTA_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW USER_PERM_DELTA_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SUSER_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM SUSER_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SUSER_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW SUSER_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'USER_PERM_SNAPSHOT_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM USER_PERM_SNAPSHOT_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'USER_PERM_SNAPSHOT_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW USER_PERM_SNAPSHOT_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SPERMISSION_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM SPERMISSION_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SPERMISSION_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW SPERMISSION_V';
  END LOOP;
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SPERMISSION_ALL_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM SPERMISSION_ALL_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SPERMISSION_ALL_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW SPERMISSION_ALL_V';
  END LOOP;
END;
/
CREATE VIEW spermission_v AS
  SELECT 
        spermission.id,
        spermission.account_grant,
        spermission.assignable,
        spermission.authoritative,
        spermission.bound_by_id,
        spermission.boundable,
        spermission.cost,
        spermission.deleted,
        spermission.description,
        spermission.display_name,
        spermission.driver_id,
        spermission.effective,
        spermission.exclude_from_catalog,
        spermission.is_curated,
        spermission.logical_id,
        spermission.merged,
        spermission.name,
        spermission.not_reviewable,
        spermission.permission_id,
        spermission.prov_attribute,
        spermission.prov_by_unique_permission_id,
        spermission.prov_native_value,
        spermission.prov_unique_driver_id,
        spermission.provisionable,
        spermission.risk,
        spermission.type,
        spermission.unique_bound_by_id,
        spermission.unique_permission_id,
        spermission.value,
        spermission.value_type,
        spermission.application,
        spermission.base_application,
        spermission.collected_entity,
        spermission.data_collection,
        spermission.effective_entity,
        spermission.sversion,
        spermission.auto_generated,
        spermission.prov_unique_permission_id,
        perm_owner.perm_owner_name
FROM 
        spermission
        LEFT JOIN (SELECT rso.permission,
                LISTAGG(TO_CHAR(suser.display_name), ' | ') WITHIN GROUP (ORDER BY suser.display_name) perm_owner_name
                FROM (SELECT DISTINCT owner_unique_id, permission
                  FROM resolved_spermission_owner) rso
                  JOIN suser ON suser.unique_user_id = rso.owner_unique_id AND suser.deleted = 0 AND suser.effective = 1
          GROUP BY rso.permission) perm_owner ON perm_owner.permission = spermission.id
WHERE 
        (exclude_from_catalog = 0 OR exclude_from_catalog IS NULL) AND effective = 1;

CREATE VIEW user_perm_delta_v AS 
 SELECT DISTINCT pchange.snapshot,
    pchange.unique_permission_id perm_qid,
    perm.name perm_name,
    old_perm.name old_perm_name,
    perm.type,
    old_perm.type old_perm_type,
    perm.description,
    old_perm.description old_perm_desc,
    perm.risk,
    old_perm.risk old_perm_risk,
    perm.value,
    old_perm.value old_perm_value,
    perm.id permission_id,
    perm.collected_entity,
    pchange.new_permission,
    pchange.old_permission,
    pchange.bound_by_id_changed,
    pchange.change_type,
    pchange.name_changed,
    pchange.description_changed,
    pchange.risk_level_changed,
    pchange.type_changed,
    pchange.value_changed,
    pchange.application,
    ds.unique_id appl_qid,
    ds.name appl_name,
    usr.unique_user_id user_qid,
    usr.last_name user_last_name,
    usr.first_name user_first_name,
    usr.idm_dn user_dn,
    usr.idm_tree_name user_idm_tree_name,
    snap.id  snap_id,
    snap.timestamp snap_timestamp
   FROM permission_change pchange
     LEFT JOIN spermission perm ON pchange.unique_permission_id = perm.unique_permission_id AND perm.collected_entity = pchange.new_permission AND perm.id = perm.collected_entity
     LEFT JOIN spermission old_perm ON perm.unique_permission_id = old_perm.unique_permission_id AND old_perm.collected_entity = pchange.old_permission AND old_perm.id = old_perm.collected_entity
     LEFT JOIN data_source ds ON perm.application = ds.id
     LEFT JOIN saccount_permission ap ON ap.spermission = perm.id
     LEFT JOIN saccount acoll ON ap.saccount = acoll.id
     LEFT JOIN saccount_user au ON au.saccount = acoll.id
     LEFT JOIN suser usr ON usr.collected_entity = au.suser
     LEFT JOIN snapshot snap ON pchange.snapshot = snap.id OR pchange.old_snapshot = snap.id OR pchange.change_type = 'REMOVED'
  WHERE snap.id = pchange.snapshot AND (perm.type != 'BoundEntitlement' OR pchange.change_type = 'REMOVED') AND (pchange.change_type = 'ADDED' OR pchange.change_type = 'REMOVED' OR pchange.name_changed = 1 OR pchange.description_changed = 1 OR pchange.risk_level_changed = 1 OR pchange.type_changed = 1 OR pchange.value_changed = 1);

CREATE VIEW suser_v AS
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    TO_CHAR(suser.first_name) first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    TO_CHAR(suser.job_title) job_title,
    TO_CHAR(suser.last_name) last_name,
    suser.location,
    suser.merged,
    TO_CHAR(suser.middle_name) middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    TO_CHAR(suser.unique_user_id) unique_user_id,
    TO_CHAR(suser.user_id) user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    TO_CHAR(suser.workforce_id) workforce_id,
    suser_email.email,
    suser.risk
  FROM suser
    LEFT JOIN (SELECT
        suser.id suser,
        suser.display_name,
        LISTAGG(TO_CHAR(suser_email.email), ' | ') WITHIN GROUP(ORDER BY email) email
      FROM suser
        JOIN suser_email ON suser.id = suser_email.suser
      WHERE suser.effective = 1 AND suser.deleted = 0  
      GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
  WHERE suser.deleted = 0 AND suser.effective = 1;


CREATE VIEW user_perm_snapshot_v AS 
  SELECT DISTINCT appl.qid appl_qid,
        ds.name appl_name,
        perm.unique_permission_id perm_qid,
        perm.name perm_name,
        perm.type,
        usr.unique_user_id user_qid,
        usr.dn user_dn,
        usr.last_name user_last_name,
        usr.first_name user_first_name,
        usr.idm_tree_name user_idm_tree_name,
        snap.id snap_id,
        snap.timestamp snap_timestamp
       FROM spermission perm
         LEFT JOIN application appl ON perm.application = appl.id
         LEFT JOIN spermission pcoll ON perm.collected_entity = pcoll.id
         LEFT JOIN saccount_permission ap ON pcoll.id = ap.spermission
         LEFT JOIN saccount acoll ON ap.saccount = acoll.id
         LEFT JOIN saccount_user au ON acoll.id = au.saccount
         LEFT JOIN saccount acct ON acoll.id = acct.collected_entity
         LEFT JOIN suser ucoll ON au.suser = ucoll.id
         LEFT JOIN suser usr ON ucoll.id = usr.collected_entity
         LEFT JOIN data_source ds ON appl.id = ds.id
         LEFT JOIN snapshot_sversions ap_snap_sv ON ap.sversion = ap_snap_sv.sversions
         LEFT JOIN snapshot_sversions au_snap_sv ON au.sversion = au_snap_sv.sversions
         JOIN snapshot snap ON ap_snap_sv.snapshot = snap.id AND au_snap_sv.snapshot = snap.id;

CREATE VIEW spermission_all_v AS
 SELECT spermission.id,
    spermission.account_grant,
    spermission.assignable,
    spermission.authoritative,
    spermission.bound_by_id,
    spermission.boundable,
    spermission.cost,
    spermission.deleted,
    spermission.description,
    spermission.display_name,
    spermission.driver_id,
    spermission.effective,
    spermission.exclude_from_catalog,
    spermission.is_curated,
    spermission.logical_id,
    spermission.merged,
    spermission.name,
    spermission.not_reviewable,
    spermission.permission_id,
    spermission.prov_attribute,
    spermission.prov_by_unique_permission_id,
    spermission.prov_native_value,
    spermission.prov_unique_driver_id,
    spermission.provisionable,
    spermission.risk,
    spermission.type,
    spermission.unique_bound_by_id,
    spermission.unique_permission_id,
    spermission.value,
    spermission.value_type,
    spermission.application,
    spermission.base_application,
    spermission.collected_entity,
    spermission.data_collection,
    spermission.effective_entity,
    spermission.sversion,
    spermission.auto_generated,
    spermission.prov_unique_permission_id
   FROM spermission;

GRANT SELECT ON igops.user_perm_delta_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM user_perm_delta_v FOR igops.user_perm_delta_v;

GRANT SELECT ON igops.suser_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM suser_v FOR igops.suser_v;

GRANT SELECT ON igops.user_perm_snapshot_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM user_perm_snapshot_v FOR igops.user_perm_snapshot_v;

GRANT SELECT ON igops.spermission_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM spermission_v FOR igops.spermission_v;

GRANT SELECT ON igops.spermission_all_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM spermission_all_v FOR igops.spermission_all_v;