DROP VIEW IF EXISTS spermission_all_v;
DROP VIEW IF EXISTS spermission_v;
DROP VIEW IF EXISTS user_perm_snapshot_v;
DROP VIEW IF EXISTS suser_v;
DROP VIEW IF EXISTS user_perm_delta_v;

CREATE VIEW spermission_all_v AS
 SELECT spermission.id,
    spermission.account_grant,
    spermission.assignable,
    spermission.authoritative,
    spermission.bound_by_id,
    spermission.boundable,
    spermission.cost,
    spermission.deleted,
    spermission.description,
    spermission.display_name,
    spermission.driver_id,
    spermission.effective,
    spermission.exclude_from_catalog,
    spermission.is_curated,
    spermission.logical_id,
    spermission.merged,
    spermission.name,
    spermission.not_reviewable,
    spermission.permission_id,
    spermission.prov_attribute,
    spermission.prov_by_unique_permission_id,
    spermission.prov_native_value,
    spermission.prov_unique_driver_id,
    spermission.provisionable,
    spermission.risk,
    spermission.type,
    spermission.unique_bound_by_id,
    spermission.unique_permission_id,
    spermission.value,
    spermission.value_type,
    spermission.application,
    spermission.base_application,
    spermission.collected_entity,
    spermission.data_collection,
    spermission.effective_entity,
    spermission.sversion,
    spermission.auto_generated,
    spermission.prov_unique_permission_id
   FROM spermission;

CREATE VIEW spermission_v AS
SELECT spermission.id,
    spermission.account_grant,
    spermission.assignable,
    spermission.authoritative,
    spermission.bound_by_id,
    spermission.boundable,
    spermission.cost,
    spermission.deleted,
    spermission.description,
    spermission.display_name,
    spermission.driver_id,
    spermission.effective,
    spermission.exclude_from_catalog,
    spermission.is_curated,
    spermission.logical_id,
    spermission.merged,
    spermission.name,
    spermission.not_reviewable,
    spermission.permission_id,
    spermission.prov_attribute,
    spermission.prov_by_unique_permission_id,
    spermission.prov_native_value,
    spermission.prov_unique_driver_id,
    spermission.provisionable,
    spermission.risk,
    spermission.type,
    spermission.unique_bound_by_id,
    spermission.unique_permission_id,
    spermission.value,
    spermission.value_type,
    spermission.application,
    spermission.base_application,
    spermission.collected_entity,
    spermission.data_collection,
    spermission.effective_entity,
    spermission.sversion,
    spermission.auto_generated,
    spermission.prov_unique_permission_id,
    perm_owner.perm_owner_name
   FROM spermission
     LEFT JOIN (SELECT rso.permission,
            string_agg(suser.display_name::text, ' | '::text) AS perm_owner_name
           FROM (SELECT DISTINCT owner_unique_id,
        permission
        FROM resolved_spermission_owner) rso
             JOIN suser ON suser.unique_user_id::text = rso.owner_unique_id::text AND suser.deleted = false AND suser.effective = true
          GROUP BY rso.permission) perm_owner ON perm_owner.permission = spermission.id
  WHERE (spermission.exclude_from_catalog = false OR spermission.exclude_from_catalog IS NULL) AND spermission.effective = true;

CREATE VIEW user_perm_snapshot_v AS 
 SELECT DISTINCT appl.qid AS appl_qid,
    ds.name AS appl_name,
    perm.unique_permission_id AS perm_qid,
    perm.name AS perm_name,
    perm.type,
    usr.unique_user_id AS user_qid,
    usr.dn AS user_dn,
    usr.last_name AS user_last_name,
    usr.first_name AS user_first_name,
    usr.idm_tree_name AS user_idm_tree_name,
    snap.id AS snap_id,
    snap."timestamp" AS snap_timestamp
   FROM spermission perm
     LEFT JOIN application appl ON perm.application = appl.id
     LEFT JOIN spermission pcoll ON perm.collected_entity = pcoll.id
     LEFT JOIN saccount_permission ap ON pcoll.id = ap.spermission
     LEFT JOIN saccount acoll ON ap.saccount = acoll.id
     LEFT JOIN saccount_user au ON acoll.id = au.saccount
     LEFT JOIN saccount acct ON acoll.id = acct.collected_entity
     LEFT JOIN suser ucoll ON au.suser = ucoll.id
     LEFT JOIN suser usr ON ucoll.id = usr.collected_entity
     LEFT JOIN data_source ds ON appl.id = ds.id
     LEFT JOIN snapshot_sversions ap_snap_sv ON ap.sversion = ap_snap_sv.sversions
     LEFT JOIN snapshot_sversions au_snap_sv ON au.sversion = au_snap_sv.sversions
     JOIN snapshot snap ON ap_snap_sv.snapshot = snap.id AND au_snap_sv.snapshot = snap.id;

CREATE VIEW suser_v AS 
SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk
FROM suser
    LEFT JOIN 
        (SELECT
            suser.id suser,
            suser.display_name,
            string_agg(suser_email.email, ' | ' order by suser_email.email) As email
        FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
        GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
WHERE suser.deleted = '0' AND suser.effective = '1';

CREATE VIEW user_perm_delta_v AS 
 SELECT DISTINCT pchange.snapshot,
    pchange.unique_permission_id AS perm_qid,
    perm.name AS perm_name,
    old_perm.name AS old_perm_name,
    perm.type,
    old_perm.type AS old_perm_type,
    perm.description,
    old_perm.description AS old_perm_desc,
    perm.risk,
    old_perm.risk AS old_perm_risk,
    perm.value,
    old_perm.value AS old_perm_value,
    perm.id AS permission_id,
    perm.collected_entity,
    pchange.new_permission,
    pchange.old_permission,
    pchange.bound_by_id_changed,
    pchange.change_type,
    pchange.name_changed,
    pchange.description_changed,
    pchange.risk_level_changed,
    pchange.type_changed,
    pchange.value_changed,
    pchange.application,
    ds.unique_id AS appl_qid,
    ds.name AS appl_name,
    usr.unique_user_id AS user_qid,
    usr.last_name AS user_last_name,
    usr.first_name AS user_first_name,
    usr.idm_dn AS user_dn,
    usr.idm_tree_name AS user_idm_tree_name,
    snap.id AS snap_id,
    snap."timestamp" AS snap_timestamp
   FROM permission_change pchange
     LEFT JOIN spermission perm ON pchange.unique_permission_id::text = perm.unique_permission_id::text AND perm.collected_entity = pchange.new_permission AND perm.id = perm.collected_entity
     LEFT JOIN spermission old_perm ON perm.unique_permission_id::text = old_perm.unique_permission_id::text AND old_perm.collected_entity = pchange.old_permission AND old_perm.id = old_perm.collected_entity
     LEFT JOIN data_source ds ON perm.application = ds.id
     LEFT JOIN saccount_permission ap ON ap.spermission = perm.id
     LEFT JOIN saccount acoll ON ap.saccount = acoll.id
     LEFT JOIN saccount_user au ON au.saccount = acoll.id
     LEFT JOIN suser usr ON usr.collected_entity = au.suser
     LEFT JOIN snapshot snap ON pchange.snapshot = snap.id OR pchange.old_snapshot = snap.id OR pchange.change_type::text = 'REMOVED'::text
  WHERE snap.id = pchange.snapshot AND (perm.type::text <> 'BoundEntitlement'::text OR pchange.change_type::text = 'REMOVED'::text) 
  AND (pchange.change_type::text = 'ADDED'::text OR pchange.change_type::text = 'REMOVED'::text OR pchange.name_changed = true OR pchange.description_changed = true OR pchange.risk_level_changed = true OR pchange.type_changed = true OR pchange.value_changed = true);


ALTER TABLE spermission_all_v
  OWNER TO igops;
GRANT ALL ON TABLE spermission_all_v TO igops;
GRANT SELECT ON TABLE spermission_all_v TO ig_report_role;

ALTER TABLE spermission_v
  OWNER TO igops;
GRANT ALL ON TABLE spermission_v TO igops;
GRANT SELECT ON TABLE spermission_v TO ig_report_role;

ALTER TABLE user_perm_snapshot_v
  OWNER TO igops;
GRANT ALL ON TABLE user_perm_snapshot_v TO igops;
GRANT SELECT ON TABLE user_perm_snapshot_v TO ig_report_role;

ALTER TABLE suser_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_v TO igops;
GRANT SELECT ON TABLE suser_v TO ig_report_role;

ALTER TABLE user_perm_delta_v 
  OWNER TO igops;
GRANT ALL ON TABLE user_perm_delta_v  TO igops;
GRANT SELECT ON TABLE user_perm_delta_v  TO ig_report_role;
